/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo       (ChoonHui.Teo@nicta.com.au)
 *                Quoc V. Le          (Quoc.Le@nicta.com.au)
 *                Alex Smola          (Alex.Smola@nicta.com.au)
 *                S.V.N. Vishwanathan (SVN.Vishwanathan@nicta.com.au)        
 *
 * Created      : 18/12/2006
 *
 * Last Updated :
 */

#include <iostream>
#include <string>
#include <vector>
#include <cstdlib>
#include <iomanip>
#include <time.h>
#include <fstream>
#include <math.h>

#include "common.hpp"
#include "core_header.hpp"
#include "loss.hpp"
//#include "inputoutput.hpp"

#include "petsc.h"


CLoss * SelectLoss(CInfo *info, CDataset *ds);

using namespace std;



int main(int argc, char **argv)
{
  CInfo      info; 
  CMatVecArr *w = 0;
  CLabel     ybar;
  CMatVecArr *f = 0;
  ofstream   fvalFp;
  CLoss      *lf = 0;
  Real       loss = 0;

  int  dummy1 = 0;
  char **dummy2;
  PetscInitialize(&dummy1, &dummy2, (char*)0, "");

  // parse command line arguments
  ParseCmdLineArgPredict(argc, argv, &info);  
  int learningType = info.learningType;  // keep this here, in case user specified a loss function for prediction

  // read dataset ds from dataset file
  // note: in case model was trained with bias feature, we read test set with bias feature appended!
  info.bias = 1;
  CDataset ds(&info);

  // read model parameter w from model file
  // note: dataset must be loaded before model because model loading routine may need
  //       some information from the dataset
  ReadModelFile(w, &info);
  if(learningType != 0)
    info.learningType = learningType;   // replace the loss function that user specified

  // instantiate the right loss object
  // note that model file should be read first as there is key information such as CInfo::k needed by
  //   some loss function
  lf = SelectLoss(&info, &ds);
  
  // compute loss function value
  lf->ComputeLoss(w, &ds, loss);

  // predict labels
  f = new CMatVecArr(ds.nExample, w->col); // f := Xw
  lf->Predict(w, &ds, &ybar, f);


  // display loss
  printf("Loss function value : %lf\n", loss);

  // display performance only when true labels are provided
  if(info.programMode != PREDICTION_MODE)
    lf->EvalPerformance(w, &ds, &ybar, f);


  // dump predictions and/or f values into files
  switch(info.outFormat) {
  case NO_OUTPUT:
    break;

  case LABEL_ONLY:
    ybar.ToFile(info.outputLabelFn);
    printf("Labels written in file : %s\n",info.outputLabelFn.c_str());
    break;

  case LABEL_AND_FVAL:
    ybar.ToFile(info.outputLabelFn);
    printf("Labels written in file : %s\n",info.outputLabelFn.c_str());
  case FVAL_ONLY:
    fvalFp.open(info.outputFValFn.c_str());
    if(!fvalFp.good()) {
      printf("ERROR: {bmrm-predict} cannot open file <%s> for pre-decision function values!\n", info.outputFValFn.c_str());
    }
    else {
      for(int i=0; i<f->row*f->col; i++)
        fvalFp << f->array[i] << endl;
    }
    fvalFp.close();
    printf("Pre-decision function values written in file : %s\n",info.outputFValFn.c_str());
    break;
  }

  // cleaning up
  delete lf;
  delete w;

  PetscFinalize();
  return 0;
}


