/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo (ChoonHui.Teo@anu.edu.au)
 * Created      : 01/01/2007
 * Last Updated :
 */


#ifndef _COMMON_CPP_
#define _COMMON_CPP_

#include "errorcode.hpp"
#include "common.hpp"
#include <cassert>
#include <memory>
#include <time.h>

using namespace std;



// we need new communicator when we have redundant machines
MPI_Group PETSC_group_world   = 0;
MPI_Group BMRM_group_world    = 0;
MPI_Comm  BMRM_COMM_WORLD     = 0;
int       *BMRM_Process_Ranks = 0;


/**
 * Check if the learning problem is a binary classification problem
 */
int IsBinaryClassification(int learningType){
  if (learningType == SOFT_MARGIN_LOSS ||
      learningType == LOGISTIC_LOSS ||
      learningType == EXPONENTIAL_LOSS ||
      learningType == HUBER_HINGE_LOSS ||
      learningType == FBETA_SCORE_LOSS ||
      learningType == ROC_SCORE_LOSS)
  {
    return 1;
  }
  return 0;
}

/** 
 * Check if the learning problem is a regression problem
 */
int IsRegression(int learningType){
  if (learningType == LEAST_SQUARES_LOSS||
      learningType == HUBER_LOSS||
      learningType == ESVR_LOSS)
    {
      return 1;
    }
  return 0;

}

/** 
 * Check if the learning problem is a ranking problem
 */
int IsRanking(int learningType){
  if (learningType == NDCG_LOSS||
      learningType == ORDINAL_REGRESSION_LOSS)
    {
      return 1;
    }
  return 0;

}


/* Comparator for descending sort */
int numeric_compare(const void *a, const void *b) 
{
  double va,vb;
  va=((SCORE_INDEX *)a)->score;
  vb=((SCORE_INDEX *)b)->score;
  if(va == vb) {
    va=((SCORE_INDEX *)a)->q;
    vb=((SCORE_INDEX *)b)->q;
  }
  return(-(va > vb) + (va < vb));
}


// struct indirect_less_than {
//   double *array;
//   indirect_less_than(double *value) {array=value;}
//   bool operator() (const int &a, const int &b) {return array[a] < array[b];}
// };


// int nnz(double *x, int n)
// {
//   int cnt = 0;

//   for(int i=0; i<n; i++)
//     if(ABS(x[i]) > ZERO_EPS) cnt++;

//   return cnt;
// }



#endif
