/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo (ChoonHui.Teo@anu.edu.au)
 * Created      : 28/12/2006
 * Last Updated :
 */

#ifndef _COMMON_HPP_
#define _COMMON_HPP_


#define INFTY     1e30
#define ZERO_EPS  1e-16
 
#define Real      PetscScalar   // i.e. double in default PETSc configuration (if you use PETSc library...)


// program mode
#define LEARNING_MODE      1    // do learning
#define PRED_AND_EVAL_MODE 2    // do prediction and evaluation (when labelset is supplied)
#define PREDICTION_MODE    3    // do prediction only (labelset is not supplied)

// dataset
//#define PATTERN Real
//#define LABEL   Real

// MatVecArr
#define ROW_MATRIX    1
#define COLUMN_MATRIX 2


// user intervention in deciding dataset type
#define AUTOMATIC    1
#define FORCE_SPARSE 2
#define FORCE_DENSE  3


// inner solver
#define DAIFLETCHER   1
#define QPLOQO        2
#define L1LP          3
#define CUTTING_PLANE 4

#define NUMBER_OF_SOLVER 4   // how many solvers we have?

// regularizer
#define L1NORM   1
#define L2NORMSQ 2


// for prLOQO
#define QUIET  1
#define STATUS 2
#define FLOOD  3

#define OPTIMAL_SOLUTION            1
#define SUBOPTIMAL_SOLUTION         2
#define ITERATION_LIMIT             3
#define PRIMAL_INFEASIBLE           4
#define DUAL_INFEASIBLE             5
#define PRIMAL_AND_DUAL_INFEASIBLE  6
#define INCONSISTENT                7
#define PRIMAL_UNBOUNDED            8
#define DUAL_UNBOUNDED              9
#define TIME_LIMIT                  10



// BMRM learning type (loss functions)
#define SOFT_MARGIN_LOSS           1
#define SQUARED_SOFT_MARGIN_LOSS   2
#define HUBER_HINGE_LOSS           3
#define LOGISTIC_LOSS              4
#define EXPONENTIAL_LOSS           5
#define ROC_SCORE_LOSS             6
#define FBETA_SCORE_LOSS           7

#define ESVR_LOSS                  8
#define HUBER_LOSS                 9
#define LEAST_SQUARES_LOSS         10
#define LEAST_DEVIATION_LOSS       11

#define QUANTILE_LOSS              12
#define NOVELTY_LOSS               13
#define POISSON_LOSS               14

#define NDCG_LOSS                  15
#define ORDINAL_REGRESSION_LOSS    16

#define NUMBER_OF_LOSS             16  // how many loss function we have



// BMRM prediction output type
#define NO_OUTPUT          0
#define LABEL_ONLY         1
#define FVAL_ONLY          2
#define LABEL_AND_FVAL     3



// useful macros
#ifndef MAX
  #define MAX(a,b) ((a)>(b) ? (a) : (b))
#endif
#ifndef MIN
  #define MIN(a,b) ((a)<(b) ? (a) : (b))
#endif
#define ABS(a)   ((a)>0.0 ? (a) : (-(a)))
#define SQR(a)   ((a)*(a))
#define SGN(a)   ((a)>0.0 ? (1.0) : (-1.0))


// for parallel bmrm
#include "petsc.h"
//#undef SEEK_SET
//#undef SEEK_END
//#undef SEEK_CUR
//#include "mpi.h"

// we need new communicator when we have redundant machines
extern MPI_Group PETSC_group_world;
extern MPI_Group BMRM_group_world;
extern MPI_Comm  BMRM_COMM_WORLD;
extern int       *BMRM_Process_Ranks;


#define SERIAL_CENTRALISED_DS    1
#define PARALLEL_CENTRALISED_DS  2
#define PARALLEL_DISTRIBUTED_DS  3

#define MYROOT                   0
#define MASTER(id)               if((id) == MYROOT)
#define ALL_SLAVES(id)           if((id) != MYROOT)
#define SOME_SLAVES(id, limit)   if(((id) != MYROOT) && ((id) <= (limit)))
#define SOME_MACHINES(id, limit) if((id) < (limit))

// for historical purpose
#define LOCAL_DATA_PARALLEL_COMPUTATION           1
#define DISTRIBUTED_DATA_DISTRIBUTED_COMPUTATION  2


int IsBinaryClassification(int);
int IsRegression(int);
int IsRanking(int);

/* Structure to keep score and index
   use for quick-sort */
typedef struct score_index {
  int index;
  double score;
  double q;
} SCORE_INDEX;

/* Comparator for descending sort */
int numeric_compare(const void *a, const void *b);


/** Functor for ascending indices sorting i.e., sort the indices based on the corresponding 
 *  values.  
 */
struct indirect_less_than {
  double *array;
  indirect_less_than(double *value) {array=value;}
  bool operator() (const int &a, const int &b) {return array[a] < array[b];}
};

/** Functor for descending indices sorting i.e., sort the indices based on the corresponding 
 *  values.  
 */
struct indirect_greater_than {
  double *array;
  indirect_greater_than(double *value) {array=value;}
  bool operator() (const int &a, const int &b) {return array[a] > array[b];}
};

#endif
