/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo       (ChoonHui.Teo@nicta.com.au)
 *                Quoc V. Le          (Quoc.Le@nicta.com.au)
 *                Alex Smola          (Alex.Smola@nicta.com.au)
 *                S.V.N. Vishwanathan (SVN.Vishwanathan@nicta.com.au)        
 *
 * Created      : 25/02/2007
 *
 * Last Updated :
 */

#ifndef _INFO_CPP_
#define _INFO_CPP_

#include "info.hpp"

/**  Constructor
 */
CInfo::CInfo()
  :programMode(LEARNING_MODE),
   convLog(0),
   learningType(SOFT_MARGIN_LOSS),
   regularizer(2),
   bias(0),
   memSize(200),
   k(-999),
   dimX(0),
   dimW(0),
   C(1),
   lambda(1),
   epsilon(0.1),
   tau(0.1),
   truncation(10),
   hostname(""),
   datasetFn(""),
   labelsetFn(""),
   dsList(""),
   datasetType(AUTOMATIC),
   modelFn(""),
   solverType(DAIFLETCHER),
   verbosity(1),
   maxIter(200),
   gammaTol(-1),
   epsilonTol(-1),
   procID(MYROOT),
   nNode(1),
   nUsedNode(1),
   compMode(SERIAL_CENTRALISED_DS),
   outputLabelFn(""),
   outputFValFn(""),
   outFormat(LABEL_ONLY)

{}


/** Print (selected) information
 */
void CInfo::Print()
{
  printf("=====  Information and parameters   =====\n");
  printf(" learningType          : %d\n",    learningType);
  printf(" regularizer           : %d\n",    regularizer);
  printf(" added bias feature?   : %s\n",    (bias) ? "Yes" : "No");
  printf(" memSize               : %d\n",    memSize);
  printf(" k (i.e. max label)    : %d\n",    k);
  printf(" dimX (data dim.)      : %d\n",    dimX);
  printf(" dimW                  : %d\n",    dimW);
  printf(" C                     : %.10f\n", C);
  printf(" lambda                : %.10f\n", lambda);
  printf(" epsilon               : %.10f\n", epsilon);
  printf(" tau                   : %.10f\n", tau);
  printf(" truncation            : %d\n",    truncation);
  printf(" Machine name          : %s\n",    hostname.c_str());
  printf(" Data file             : %s\n",    datasetFn.c_str());
  printf(" Label file            : %s\n",    labelsetFn.c_str());
  printf(" Dataset List          : %s\n",    dsList.c_str());
  printf(" Model file            : %s\n",    modelFn.c_str());
  printf(" Solver                : %d\n",    solverType);
  printf(" verbosity             : %d\n",    verbosity);
  printf(" Max. Col. Gen. Iter   : %d\n",    maxIter);
  
  double tol = epsilonTol;
  string g_or_e = "(epsilon)";
  if(gammaTol > 0) {
    tol = gammaTol;
    g_or_e = "(gamma)";
  }
  printf(" Tolerance %9s : %.8f\n",          g_or_e.c_str(), tol);
  printf(" # of avail. machines  : %d\n",    nNode);
  printf(" # of machines used    : %d\n",    nUsedNode);
  printf(" Computation mode      : %d\n",    compMode);
  printf("=========================================\n");
}
   
#endif
