/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo       (ChoonHui.Teo@nicta.com.au)
 *                Quoc V. Le          (Quoc.Le@nicta.com.au)
 *                Alex Smola          (Alex.Smola@nicta.com.au)
 *                S.V.N. Vishwanathan (SVN.Vishwanathan@nicta.com.au)        
 *
 * Created      : 25/02/2007
 *
 * Last Updated :
 */

#ifndef _INFO_HPP_
#define _INFO_HPP_

#include "common.hpp"
#include <string>

using namespace std;

/**  Class for information needed by BMRM
 *   This Class contain both learning and miscellaneous information.
 */
class CInfo {

public:
  
  int    programMode;    // learning or prediction? 
  int    convLog;        // log convergence statistics into files?

  // problem specific (extend this if users have new paramters for new loss functions)
  int    learningType;   // what learning problem
  int    regularizer;    // L1 or L2 norm
  int    bias;           // applied shifted hyperplane? (default : yes)
  int    memSize;        // maximum size of the set W (keep most recent ones)
  int    k;              // number of class (default) or the dimensionality of multiple label.
                         //   user may change this.  
  int    dimX;           // dimension of vectorial Pattern, X
  int    dimW;           // dimension of weight vector
  double C;              // slack trade off parameter
  double lambda;         // regularizer scaling factor
  double epsilon;        // for epsilon-insensitive svr
  double tau;            // for quantile regression
  int    truncation;     // NDCG, MRR

  // miscellaneous
  string hostname;       // Machine name
  string datasetFn;      // Dataset (data only) filename  
  string labelsetFn;     // labelset filename
  string dsList;         // datasets file list (for distributed DS case)
  int    datasetType;    // {AUTOMATIC, FORCE_SPARSE, FORCE_DENSE}
  string modelFn;        // Model filename
  int    solverType;     // which inner solver
  int    verbosity;      // program verbosity level
  int    maxIter;        // maximum qp iteration
  double gammaTol;       // tolerance for the gamma-termination criterion see BMRM paper
  double epsilonTol;     // tolerance for the epsilon-termination criterion see Joachims' KDD06

  // parallel and distributed computation related
  int    procID;         // processor id of current machine
  int    nNode;          // number of machines running bmrm:
                         //   1 := serial bmrm; >1 := parallel bmrm
  int    nUsedNode;      // in cases where nNode > nSample, only a portion of the 
                         //   available machines will be used.   
  int    compMode;       // computation mode: {serial, parallel/distributed}
                         //   1 := serial (with centralised dataset)
                         //   2 := parallel with centralised dataset
                         //   3 := parallel with distributed dataset  

  // for prediction phase
  string outputLabelFn;       // predicted label output filename
  string outputFValFn;        // pre-decision function values output filename
  int    outFormat;           // prediction output format
  
  CInfo();
  virtual ~CInfo() {}

  void Print();

};

#endif
