/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo       (ChoonHui.Teo@nicta.com.au)
 *                Quoc V. Le          (Quoc.Le@nicta.com.au)
 *                Alex Smola          (Alex.Smola@nicta.com.au)
 *                S.V.N. Vishwanathan (SVN.Vishwanathan@nicta.com.au)        
 *
 * Created      : 26/02/2007
 *
 * Last Updated :
 */

#ifndef _MATVECARR_HPP_
#define _MATVECARR_HPP_

#include "errorcode.hpp"
#include "common.hpp"
#include "petsc.h"
#include "petscmat.h"
#include "petscvec.h"


/** Container for PetscScalar*, Vec and Mat referencing to the same content.
 */
class CMatVecArr {

public:
  int  row;       // # of row
  int  col;       // # of column

  Real *array;    // the values
  Vec  vec;       // vector representation; e.g. a row*col length vector
  Mat  mat;       // matrix representation; e.g. a row-by-col matrix
  Mat  matT;      // the tranposed mat
    
  CMatVecArr(int _row, int _col, Real* _array=0):row(_row), col(_col) 
  {
    // array
    if(_array == 0)
      array = (Real*)calloc(row*col, sizeof(Real));  // zero entries      
    else {
      array = (Real*)malloc(row*col*sizeof(Real));
      memcpy(array, _array, row*col*sizeof(Real));
    }
          
    // PETSc vec
    VecCreateSeqWithArray(PETSC_COMM_SELF, row*col, array, &vec);
    VecAssemblyBegin(vec);
    VecAssemblyEnd(vec);

    // PETSC mat
    MatCreateSeqDense(PETSC_COMM_SELF, row, col, array, &mat);
    MatAssemblyBegin(mat, MAT_FINAL_ASSEMBLY); 
    MatAssemblyEnd(mat, MAT_FINAL_ASSEMBLY);

    // PETSC transposed mat
    MatCreateSeqDense(PETSC_COMM_SELF, col, row, array, &matT);
    MatAssemblyBegin(matT, MAT_FINAL_ASSEMBLY); 
    MatAssemblyEnd(matT, MAT_FINAL_ASSEMBLY);
  }

  virtual ~CMatVecArr()
  {
    //MatDestroy(mat);
    //VecDestroy(vec);
    if(array) { free(array); array = 0; }    
  }  
};

#endif
