/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo       (ChoonHui.Teo@nicta.com.au)
 *                Quoc V. Le          (Quoc.Le@nicta.com.au)
 *                Alex Smola          (Alex.Smola@nicta.com.au)
 *                S.V.N. Vishwanathan (SVN.Vishwanathan@nicta.com.au)        
 *
 * Created      : 13/12/2006
 *
 * Last Updated :
 */

#ifndef _ESVRLOSS_CPP_
#define _ESVRLOSS_CPP_


#include "esvrloss.hpp"


/**  Compute loss. 
 *   
 *   \param m    [read]  Dimensionality of f and y (# of examples)
 *   \param f    [read]  f := X w, where X is data matrix and w is current solution
 *   \param y    [read]  labels
 *   \param loss [write] Loss value computed
 */
void CESVRLoss::compute_loss(int m, Real *f, Real *y, Real &loss)
{
  for(int i=0; i < m; i++)
    loss +=  MAX(0, ABS(y[i]-f[i]) - epsilon);
}



/**  Compute loss and gradient together. 
 *   
 *   \param m    [read]  Dimensionality of f, y and g (# of examples)
 *   \param f    [read]  f := X w, where X is data matrix and w is current solution
 *   \param y    [read]  labels
 *   \param loss [write] Loss value computed
 *   \param g    [write] Gradient computed w.r.t. f (memory pre-allocated dense matrix)
 */
void CESVRLoss::compute_loss_gradient(int m, Real *f, Real *y, Real &loss, Real *g)
{
  Real loss_i = 0;
  for(int i=0; i < m ; i++) {
    // loss
    loss_i = MAX(0.0, ABS(y[i]-f[i]) - epsilon);
    loss += loss_i;    

    // gradient
    if(loss_i > 0)
      g[i] = SGN(f[i]-y[i]); 
    else
      g[i] = 0.0;
  }
}

#endif
