/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo       (ChoonHui.Teo@nicta.com.au)
 *                Quoc V. Le          (Quoc.Le@nicta.com.au)
 *                Alex Smola          (Alex.Smola@nicta.com.au)
 *                S.V.N. Vishwanathan (SVN.Vishwanathan@nicta.com.au)        
 *
 * Created      : 12/04/2007
 *
 * Last Updated :
 */

#ifndef _FBETASCORELOSS_HPP_
#define _FBETASCORELOSS_HPP_

#include "common.hpp"
#include "info.hpp"
#include "dataset.hpp"
#include "scalarclassificationloss.hpp"



/** Class for binary F_beta loss function
 *  Reference: T. Joachims, "A Support Vector Method for Multivariate Performance Measures", ICML'05.
 */
class CFBetaScoreLoss : public CScalarClassificationLoss {

protected:
  vector<int> idx;       // used in argument sorting
  vector<int> idx_pn;    // vector storing indices for +ve and -ve w*x values : first m_pos indices correspond to y==1 and the rest y==-1
  int         *p;        // pointer to beginning of i^+ vector
  int         *n;        // pointer to beginning of i^- vector
  int         m_neg;     // number of positive examples
  int         m_pos;     // number of negative examples
  Real        sum_f_pos; // sum of f_i where y_i = 1
  Real        sum_f_neg; // sum of f_i where y_i = -1
  Real        beta;      // the beta parameter
  Real        beta_sq;   // square of beta

public:
   
  // Constructor
  CFBetaScoreLoss(CInfo *info, CDataset *ds);

  /// Destructor
  virtual ~CFBetaScoreLoss() {}

  /// Methods
  virtual void compute_loss(int m, Real *f, Real *y, Real &loss);
  virtual void compute_loss_gradient(int m, Real *f, Real *y, Real &loss, Real *l);
  void delta(const int &TP, const int &TN, const int &FP, const int &FN, Real &value);
};

#endif
