/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo       (ChoonHui.Teo@nicta.com.au)
 *                Quoc V. Le          (Quoc.Le@nicta.com.au)
 *                Alex Smola          (Alex.Smola@nicta.com.au)
 *                S.V.N. Vishwanathan (SVN.Vishwanathan@nicta.com.au)        
 *
 * Created      : 13/12/2006
 *
 * Last Updated :
 */

#ifndef _HUBERHINGELOSS_HPP_
#define _HUBERHINGELOSS_HPP_

#include "common.hpp"
#include "info.hpp"
#include "dataset.hpp"
#include "scalarclassificationloss.hpp"



/** Class for Huber-Hinge loss function, with parameter h (typically \in [0.01, 0.5]).
 *  Reference : O. Chaplle, "Training a Support Vector Machine in the Primal", Neural Computation 2007.
 *
 *            / 0                       if yf > 1 + tau
 *  loss_i = <  ((1+tau-yf)^2)/(4*tau)  if |1 - yf| <= tau
 *            \ 1 - yf                  if yf < 1 - tau
 * 
 *  where x: feature, y: label, f := <w,x>, and w solution.
 */
class CHuberHingeLoss : public CScalarClassificationLoss {

protected:
  Real tau;

public:
   
  // Constructor
  CHuberHingeLoss(CInfo *info, CDataset *ds) : CScalarClassificationLoss(info, ds) { tau = info->tau; }

  /// Destructor
  virtual ~CHuberHingeLoss() {}

  /// Methods
  virtual void compute_loss(int m, Real *f, Real *y, Real &loss);
  virtual void compute_loss_gradient(int m, Real *f, Real *y, Real &loss, Real *g);
};

#endif
