/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo       (ChoonHui.Teo@nicta.com.au)
 *                Quoc V. Le          (Quoc.Le@nicta.com.au)
 *                Alex Smola          (Alex.Smola@nicta.com.au)
 *                S.V.N. Vishwanathan (SVN.Vishwanathan@nicta.com.au)        
 *
 * Created      : 13/12/2006
 *
 * Last Updated :
 */

#ifndef _LOSS_HPP_
#define _LOSS_HPP_

#include "errorcode.hpp"
#include "common.hpp"
#include "dataset.hpp"
#include "matvecarr.hpp"
#include "info.hpp"
#include "petsc.h"
#include "petscmat.h"
#include <cassert>


/** Abstract class for various type of loss functions.
 */

class CLoss {
  
protected:
  CInfo *_info;
  CDataset *_ds;

public:
  
  /// Constructor
  CLoss() {}
  CLoss(CInfo *info, CDataset *ds): _info(info), _ds(ds) {}

  /// Destructor
  virtual ~CLoss() {}

  /// Methods
  // compute loss 
  virtual ErrorCode ComputeLoss(CMatVecArr *w, CDataset *ds, Real &loss)=0;

  // compute loss and gradient together
  virtual ErrorCode ComputeLossGradient(CMatVecArr *w, CDataset *ds, Real &loss, CMatVecArr *grad)=0;

  // predict best labels
  virtual ErrorCode Predict(CMatVecArr *w, CDataset *ds, CLabel *ybar, CMatVecArr *f)=0;

  // evaluate the goodness of the predicted labels
  virtual ErrorCode EvalPerformance(CMatVecArr *w, CDataset *ds, CLabel *ybar, CMatVecArr *f)=0;
  virtual ErrorCode EvalPerformance(CMatVecArr *w, CDataset *ds, CLabel *ybar, CMatVecArr *f, vector<string> &metric, vector<double> &score)=0;
  
  
};

#endif
