/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo       (ChoonHui.Teo@nicta.com.au)
 *                Quoc V. Le          (Quoc.Le@nicta.com.au)
 *                Alex Smola          (Alex.Smola@nicta.com.au)
 *                S.V.N. Vishwanathan (SVN.Vishwanathan@nicta.com.au)        
 *
 * Created      : 18/04/2007
 *
 * Last Updated :
 */

#ifndef _LOSS_SWITCH_HPP_
#define _LOSS_SWITCH_HPP_

#include "common.hpp"
#include "loss_header.hpp"
#include "dataset.hpp"
#include "info.hpp"


/** Select loss function according to user input.
 *
 *  \param info [read] learning & miscllaneous parameters
 *  \param ds   [read] dataset 
 */
CLoss* SelectLoss(CInfo *info, CDataset *ds)
{
  switch(info->learningType) {
  case SOFT_MARGIN_LOSS:            return new CSoftMarginLoss(info, ds);
  case SQUARED_SOFT_MARGIN_LOSS:    return new CSquaredSoftMarginLoss(info, ds);
  case HUBER_HINGE_LOSS:            return new CHuberHingeLoss(info, ds); 
  case LOGISTIC_LOSS:               return new CLogisticLoss(info, ds); 
  case EXPONENTIAL_LOSS:            return new CExponentialLoss(info, ds); 
  case ROC_SCORE_LOSS:              return new CROCScoreLoss(info, ds); 
  case FBETA_SCORE_LOSS:            return new CFBetaScoreLoss(info, ds); 

  case ESVR_LOSS:                   return new CESVRLoss(info, ds);
  case HUBER_LOSS:                  return new CHuberLoss(info, ds); 
  case LEAST_SQUARES_LOSS:          return new CLeastSquaresLoss(info, ds); 
  case LEAST_DEVIATION_LOSS:        return new CLeastDeviationLoss(info, ds); 
  case QUANTILE_LOSS:               return new CQuantileLoss(info, ds); 
  case NOVELTY_LOSS:                return new CNoveltyLoss(info, ds);
  case POISSON_LOSS:                return new CPoissonLoss(info, ds);
 
  case NDCG_LOSS:                   return new CNDCGLoss(info, ds); 
  case ORDINAL_REGRESSION_LOSS: return new COrdinalRegressionLoss(info, ds);
  default: printf("\nERROR : Loss function (%d) does not exist !\n", info->learningType); 
    exit(EXIT_FAILURE);
  }  
  return 0;
}


#endif
