/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo       (ChoonHui.Teo@nicta.com.au)
 *                Quoc V. Le          (Quoc.Le@nicta.com.au)
 *                Alex Smola          (Alex.Smola@nicta.com.au)
 *                S.V.N. Vishwanathan (SVN.Vishwanathan@nicta.com.au)        
 *
 * Created      : 12/04/2007
 *
 * Last Updated :
 */

#ifndef _NDCGLOSS_HPP_
#define _NDCGLOSS_HPP_

#include "common.hpp"
#include "info.hpp"
#include "dataset.hpp"
#include "scalarrankingloss.hpp"



/** Class for NDCG ranking
 */
class CNDCGLoss : public CScalarRankingLoss {

protected:
  int truncation;
  vector< vector<int> > sort_vectors;
  int counter;
  int num_query;
  int sort_index;
  vector<int> current;
public:
   
  // Constructor
  CNDCGLoss(CInfo *info, CDataset *ds);

  /// Destructor
  virtual ~CNDCGLoss() {}

  /// Methods
  virtual void compute_loss(int m, Real *f, Real *y, Real &loss);
  virtual void compute_loss_gradient(int m, Real *f, Real *y, Real &loss, Real *l);
  
  void find_permutation(int m, Real *a, Real *b, Real *c, Real *f, int *pi);
  void compute_coefficients(int m, Real *y, Real *a, Real *b);
  void delta(int m, double *a, double *b, int *pi, double &value);
  void dcg(int m, Real *y, Real &dy);
  double get(Real *y, int index);
  void add(Real *y, int i, Real value);
};

#endif
