/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo       (ChoonHui.Teo@nicta.com.au)
 *                Quoc V. Le          (Quoc.Le@nicta.com.au)
 *                Alex Smola          (Alex.Smola@nicta.com.au)
 *                S.V.N. Vishwanathan (SVN.Vishwanathan@nicta.com.au)        
 *
 * Created      : 13/12/2006
 *
 * Last Updated :
 */

#ifndef _QUANTILELOSS_HPP_
#define _QUANTILELOSS_HPP_

#include "common.hpp"
#include "info.hpp"
#include "dataset.hpp"
#include "scalarregressionloss.hpp"



/** Class for Quantile Regression loss function
 *  loss_i = max(tau*(y_i-f_i), (1-tau)*(f_i-y_i))
 *  where f_i := <w,x_i>
 *
 *  Reference: I. Takeuchi, Q. V. Le, T. Sears, A. J. Smola, "Nonparametric quantile estimation", JMLR, 2006.
 */
class CQuantileLoss : public CScalarRegressionLoss {

protected:
  Real tau;

public:
   
  // Constructor
  CQuantileLoss(CInfo *info, CDataset *ds) : CScalarRegressionLoss(info, ds) {tau = info->tau;}

  /// Destructor
  virtual ~CQuantileLoss() {}

  /// Methods
  virtual void compute_loss(int m, Real *f, Real *y, Real &loss);
  virtual void compute_loss_gradient(int m, Real *f, Real *y, Real &loss, Real *g);
};

#endif
