/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo       (ChoonHui.Teo@nicta.com.au)
 *                Quoc V. Le          (Quoc.Le@nicta.com.au)
 *                Alex Smola          (Alex.Smola@nicta.com.au)
 *                S.V.N. Vishwanathan (SVN.Vishwanathan@nicta.com.au)        
 *
 * Created      : 12/04/2007
 *
 * Last Updated :
 */

#ifndef _ROCSCORELOSS_HPP_
#define _ROCSCORELOSS_HPP_

#include "common.hpp"
#include "info.hpp"
#include "dataset.hpp"
#include "scalarclassificationloss.hpp"



/** Class for binary ROC Score (aka Area Under ROC Curve) loss function
 *  Reference: T. Joachims, "A Support Vector Method for Multivariate Performance Measures", ICML'05.
 */
class CROCScoreLoss : public CScalarClassificationLoss {

protected:
  vector<int> idx;       // used in argument sorting
  int         m_neg;
  int         m_pos;
  int         l_neg;
  int         l_pos;
  Real        scalingfactor;

public:
   
  // Constructor
  CROCScoreLoss(CInfo *info, CDataset *ds);

  /// Destructor
  virtual ~CROCScoreLoss() {}

  /// Methods
  virtual void compute_loss(int m, Real *f, Real *y, Real &loss);
  virtual void compute_loss_gradient(int m, Real *f, Real *y, Real &loss, Real *l);
};

#endif
