/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo       (ChoonHui.Teo@nicta.com.au)
 *                Quoc V. Le          (Quoc.Le@nicta.com.au)
 *                Alex Smola          (Alex.Smola@nicta.com.au)
 *                S.V.N. Vishwanathan (SVN.Vishwanathan@nicta.com.au)        
 *
 * Created      : 13/12/2006
 *
 * Last Updated :
 */

#ifndef _SCALARLOSS_HPP_
#define _SCALARLOSS_HPP_

#include "loss.hpp"
#include "timer.hpp"

/** Abstract class for scalar loss.
 *
 *  loss = \frac{1}{m} \sum_{i=1}^m l(x_i,y_i,w)
 *  where m : #examples
 *        w : model parameter
 *        (x_i,y_i) : i-th example (feature vector,label)
 *
 *  Notes: 
 *  1. User should implement the l() function by subclassing this class.
 *  2. This class does the 1/m scaling for the user!      
 */
class CScalarLoss : public CLoss {

protected:
  // local member variable
  CMatVecArr     *MVA_f;        // _f := X w
  CMatVecArr     *MVA_g;        // _g := \partial_f loss
  //Mat sparse_gT;
  PetscErrorCode ierr;          // PETSc error code
  
public:
  
  CTimer rightmult;
  CTimer leftmult;
  CTimer lossgradient;

  CScalarLoss(CInfo *info, CDataset *ds);

  virtual ~CScalarLoss();

  // derive from parent
  virtual ErrorCode ComputeLoss(CMatVecArr *w, CDataset *ds, Real &loss);
  virtual ErrorCode ComputeLossGradient(CMatVecArr *w, CDataset *ds, Real &loss, CMatVecArr *grad);

  // interfaces user should implement
  virtual void compute_loss_gradient(int m, Real *f, Real *y, Real &loss, Real *g)=0;
  virtual void compute_loss(int m, Real *f, Real *y, Real &loss)=0;

  // methods user might need to override or overload
  virtual void x_dot_w(CMatVecArr *w, CDataset *ds, Real *f);
  
};

#endif
