/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo       (ChoonHui.Teo@nicta.com.au)
 *                Quoc V. Le          (Quoc.Le@nicta.com.au)
 *                Alex Smola          (Alex.Smola@nicta.com.au)
 *                S.V.N. Vishwanathan (SVN.Vishwanathan@nicta.com.au)        
 *
 * Created      : 10/04/2007
 *
 * Last Updated :
 */

#ifndef _SCALARRANKINGLOSS_HPP_
#define _SCALARRANKINGLOSS_HPP_

#include "scalarloss.hpp"
#include <vector>
#include <string>

using namespace std;

/** Abstract class for scalar ranking loss.
 *  Mainly for the implementation of 
 *  1. decision function, and
 *  2. performance evaluation
 */
class CScalarRankingLoss : public CScalarLoss {

 
public:
  // constructor
  CScalarRankingLoss(CInfo *info, CDataset *ds) : CScalarLoss(info, ds) {}
  
  // destructor
  virtual ~CScalarRankingLoss() {}
  
  ErrorCode Predict(CMatVecArr *w, CDataset *ds, CLabel *ybar, CMatVecArr *f);
  ErrorCode EvalPerformance(CMatVecArr *w, CDataset *ds, CLabel *ybar, CMatVecArr *f);
  ErrorCode EvalPerformance(CMatVecArr *w, CDataset *ds, CLabel *ybar, CMatVecArr *f, 
			    vector<string> &metric, vector<double> &score);
  
  /// Methods
  virtual void compute_loss(int m, Real *f, Real *y, Real &loss) = 0;
  virtual void compute_loss_gradient(int m, Real *f, Real *y, Real &loss, Real *l) = 0;

  // derive from parent
  virtual ErrorCode ComputeLoss(CMatVecArr *w, CDataset *ds, Real &loss);
  virtual ErrorCode ComputeLossGradient(CMatVecArr *w, CDataset *ds, Real &loss, CMatVecArr *grad);

  // auxiliary functions to Predict() and EvalPerformance()'s : user might want to override
  virtual void predict(CMatVecArr *w, CDataset *ds, Real *ybar, Real *arg_f);
  virtual void eval_ndcg(int m, Real *f, Real *y, double &ndcg);
  
};

#endif
