/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo (ChoonHui.Teo@anu.edu.au)
 *                Quoc V. Le (quoc.le@anu.edu.au)
 * Created      : 18/12/2006
 * Last Updated :
 */

#ifndef _STATISTICS_HPP_
#define _STATISTICS_HPP_

#include <math.h>
#include "common.hpp"

/** Class for computing mean, variance, and some statistics
 *
 */
class CStatistics{
public:
  CStatistics(){
    sum = 0.0;
    sum_sq = 0.0;
    counter = 0;
  };
  void Add(double value);
  void Add(vector<double> values);
  double GetMean();
  double GetVariance();
  double GetStandardDeviation();
  void Reset();
private:
  double sum;
  double sum_sq;
  int counter;
};

/** Get the distribution mean (average).
 *
 */
double
CStatistics::GetMean(){
  return 1.0*sum/counter;
}

/** Add a vector to compute the statistics.
 *  \param values [read] vector of double numbers
 */
void 
CStatistics:: Add(vector<double> values){
  for (unsigned int i=0;i<values.size();i++){
    Add(values[i]);
  }
}

/** Add a number to compute the statistics.
 * \param number [read] a double number.
 */
void
CStatistics::Add(double number){
  sum += number;
  sum_sq += number*number;
  counter += 1;
}

/** Get the variance of the distribution.
 * 
 */
double
CStatistics::GetVariance(){
  return sum_sq/(counter-1) - sum*sum/((counter-1)*counter);
}

/** Get the standard deviation of the distribution.
 * 
 */
double
CStatistics::GetStandardDeviation(){
  return sqrt(GetVariance());

}

/** Reset all the iternal parameters (useful for recomputing mean 
 * variance, standard deviation of a new distributions).
 */
void 
CStatistics:: Reset(){
  sum_sq = 0.0;
  sum = 0.0;
  counter = 0;
}

#endif
