/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo       (ChoonHui.Teo@nicta.com.au)
 *                Quoc V. Le          (Quoc.Le@nicta.com.au)
 *                Alex Smola          (Alex.Smola@nicta.com.au)
 *                S.V.N. Vishwanathan (SVN.Vishwanathan@nicta.com.au)        
 *
 * Created      : 30/04/2007
 * Last Updated :
 * 
 * Purpose      : Linear Program solver using COIN-OR Clp for cutting plane method
 *
 */

#ifndef _CUTTINGPLANE_HPP_
#define _CUTTINGPLANE_HPP_


#include "errorcode.hpp"
#include "common.hpp"
#include "info.hpp"
#include "matvecarr.hpp"
#include "solver.hpp"

// from COIN-OR Clp
#include "ClpSimplex.hpp"

// from PETSc
#include "petsc.h"
#include "petscvec.h"


class CCuttingPlane : public CSolver
{
public:
  
  CCuttingPlane() {}
  CCuttingPlane(CInfo* _info, Real _tol=0);
  
  virtual ~CCuttingPlane();
  
  ErrorCode Solve();
  ErrorCode Update(CMatVecArr* a, Real b);
  ErrorCode GetSolution(CMatVecArr* &w, Real &xi, Real &regval, Real &objval);
  
private:
  ClpSimplex *sim;
  int        *newRowIndices;
  Real       *newRowElements;
  
};

#endif
