/* Copyright (c) 2006, National ICT Australia
 * All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Authors      : Choon Hui Teo (ChoonHui.Teo@anu.edu.au)
 * Created      : 07/02/2007
 * Last Updated :
 * 
 * Purpose      : L1-norm Linear Program solver using COIN-OR Clp 
 *
 */

#ifndef _L1CLP_HPP_
#define _L1CLP_HPP_


#include "errorcode.hpp"
#include "common.hpp"
#include "info.hpp"
#include "matvecarr.hpp"
#include "solver.hpp"

// from COIN-OR Clp
#include "ClpSimplex.hpp"

// from PETSc
#include "petsc.h"
#include "petscvec.h"

/* 
 * solve the 1-norm linear program
 *
 * minimize    c' * x + |x|_1
 * subject to  r <= A*x <= r+b
 *             l <= x <= u
 *
 */

class CL1Clp : public CSolver
{
public:
  
  CL1Clp() {}
  CL1Clp(CInfo* _info, Real _tol=0);
  
  virtual ~CL1Clp();
  
  ErrorCode Solve();
  ErrorCode Update(CMatVecArr* a, Real b);
  ErrorCode GetSolution(CMatVecArr* &w, Real &xi, Real &regval, Real &objval);
  
private:
  ClpSimplex *sim;
  int        *newRowIndices;
  Real       *newRowElements;
  
};

#endif
